/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: Clock.java 993 2006-09-04 08:13:08Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.engine.clock;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.component.ServiceUnitManager;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.logging.ComponentLogger;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Node;

/**
 * @author alouis
 */
public class Clock implements Component, ComponentLifeCycle {

    private static final String SERVICE = "ClockService";

    private static final String ENDPOINT = "ClockEndpoint";

    private static final QName SERVICE_NAME = new QName(
        "http://127.0.0.1:8080/axis/Clock.jws", SERVICE);

    private ComponentContext context;

    private DeliveryChannel channel;

    private ServiceEndpoint endpointReference;

    private ClockListener listener;

    private Logger logger;

    private Map<String, TimeZonesMap> timeZoneMaps;

    public ObjectName getExtensionMBeanName() {
        return null;
    }

    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    public Document getServiceDescription(ServiceEndpoint arg0) {
        Document doc = null;
        if (SERVICE_NAME.equals(arg0.getServiceName())) {
            File wsdlFile = new File(context.getInstallRoot()
                + File.separator + "META-INF" + File.separator
                + "ClockService.wsdl");
            try {
                doc = WSDLHelper.createDocumentFromWSDL(wsdlFile);
            } catch (Exception e) {
                logger.throwing(Clock.class.getName(), "getServiceDescription",
                    e);
            }
            return WSDLHelper.createDocumentFromWSDL(wsdlFile);
        }
        return doc;
    }

    public ServiceUnitManager getServiceUnitManager() {
        return new ClockServiceUnitManager(this);
    }

    public void init(ComponentContext ctx) throws JBIException {
        this.context = ctx;
        Logger log = ctx.getLogger("", null);
        logger = new ComponentLogger(log, log.getName(), log
            .getResourceBundleName(),context.getComponentName());

        logger.log(Level.INFO, "init");

        timeZoneMaps = new HashMap<String, TimeZonesMap>();
    }

    public boolean isExchangeWithConsumerOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        logger.log(Level.INFO, "accept the exchange");
        return true;
    }

    public boolean isExchangeWithProviderOkay(ServiceEndpoint arg0,
        MessageExchange arg1) {
        return false;
    }

    public ServiceEndpoint resolveEndpointReference(DocumentFragment arg0) {
        ServiceEndpoint result = null;

        if (arg0 != null) {
            Node service = XMLHelper.findChild(arg0, "service", true);

            if (service != null) {
                String serviceRequested = XMLHelper.getAttributeValue(service,
                    "name");

                if (SERVICE.equals(serviceRequested)) {
                    result = endpointReference;
                }
            }
        }
        return result;
    }

    public void shutDown() throws JBIException {
        logger.log(Level.INFO, "shutDown");

        this.listener.stopProcessing();
    }

    public void start() throws JBIException {
        logger.log(Level.INFO, "start");

        try {
            this.channel = this.context.getDeliveryChannel();
            this.listener = new ClockListener(this.channel, logger);

            Thread listenerThread = new Thread(this.listener);
            listenerThread.start();

            endpointReference = this.context.activateEndpoint(SERVICE_NAME,
                ENDPOINT);
        } catch (MessagingException e) {
            throw new JBIException(e);
        }

    }

    public void stop() throws JBIException {
        logger.log(Level.INFO, "stop");

        this.listener.stopProcessing();
    }

    protected void addTimeZonesForArtifact(String artifactName, TimeZonesMap tzm) {
        timeZoneMaps.put(artifactName, tzm);
    }

    protected int getOffSet(String city) {
        Integer tz = null;
        for (TimeZonesMap tzm : timeZoneMaps.values()) {
            tz = tzm.getTimeZone(city);
        }
        if (tz != null) {
            return tz.intValue();
        } else {
            return 0;
        }
    }

    protected TimeZonesMap getTimeZonesForArtifact(String artifactName) {
        return timeZoneMaps.get(artifactName);
    }

    protected void removeTimeZonesForArtifact(String artifactName) {
        timeZoneMaps.remove(artifactName);
    }

    public ComponentContext getContext() {
        return context;
    }
}
