/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: HelloworldProcessor.java 12:18:02 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.helloworld;

import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.listener.AbstractInternalMEProcessor;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;
import org.objectweb.petals.component.common.xmlmanipulation.StringHelper;

/**
 * This class is the message processor for the Helloworld component
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 * @author ddesjardins - eBMWebsourcing
 */
public class HelloworldProcessor extends AbstractInternalMEProcessor {

    public HelloworldProcessor(DeliveryChannel c, Logger l) {
        super(c, l);
    }

    @Override
    protected void sendInMessage(QName service, QName operation,
        NormalizedMessage in) throws Exception {
        if (new QName("printMessage").equals(operation)) {
            String content = SourceHelper.createString(in.getContent());
            if (StringHelper.isEmpty(content)) {
                l.log(Level.WARNING, "hello.printNothing");
                // this will be automaticly added to a Fault if the message
                throw new Exception(
                    "printMessage operation : you told me nothing !!");
            }
            l.log(Level.INFO, "hello.received", new String[] {content});

        } else {
            l.log(Level.WARNING, "Helloworld : incompatible operation/MEP");
            // this will be automaticly added to a Fault if the message is
            // Robust InOnly
            throw new Exception(
                "Only 'printMessage' operation is recognized for an InOnly message");
        }
    }

    @Override
    protected boolean sendInOutMessage(QName service, QName operation,
        NormalizedMessage in, NormalizedMessage out, boolean optionalOut)
        throws Exception {
        if (new QName("sayHello").equals(operation)) {
            String content = SourceHelper.createString(in.getContent());
            l.log(Level.INFO, "hello.received", new String[] {content});
            if (StringHelper.isEmpty(content)) {
                l.log(Level.WARNING, "hello.sayNothing");
                // this will be automaticly added to a Fault if the message
                throw new Exception(
                    "sayHello operation : you told me nothing !!");
            } else {
                ResourceBundle.getBundle("hello").getString("hello");
                String response = "<sayHelloResponse xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" "
                    + "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"><sayHelloReturn xsi:type=\"xsd:string\">"
                    + ResourceBundle.getBundle("hello").getString("hello")
                    + " " + content + "' </sayHelloReturn></sayHelloResponse>";
                l.log(Level.INFO, "hello.answer", new String[] {response});
                out.setContent(SourceHelper.createSource(response));
                return true;
            }
        } else {
            l.log(Level.WARNING, "hello.incomp");
            // this will be automaticly added to a Fault if the message is
            // Robust InOnly
            throw new Exception(
                "Only 'sayHello' operation is recognized for an InOut message");
        }
    }

}
