/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: OutgoingJmsServiceUnitHandler.java 601 2006-06-13 12:53:50Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms.outgoing;

import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.binding.jms.JbiConnectionFactory;
import org.objectweb.petals.binding.jms.JmsConnection;
import org.objectweb.petals.binding.jms.JmsConnectionFactory;
import org.objectweb.petals.binding.jms.incoming.JbiConnection;
import org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.ManagementMessageUtil;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.objectweb.petals.component.common.xmlmanipulation.XMLHelper;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptor;
import org.objectweb.petals.tools.jbicommon.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.tools.jbicommon.descriptor.Services;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import static org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper.createDocumentFromWSDL;

/**
 * This class is used to deploy external Queue or Topic as service
 * 
 * @version $Revision: $ $Date: $
 * @since Petals 1.0
 * @author alouis
 */
public class OutgoingJmsServiceUnitHandler extends PetalsServiceUnitHandler {

    protected HashMap<String, JmsConnection> jmsConnexions;

    protected HashMap<String, String> suRootPaths;

    protected HashMap<String, Document> descriptions;

    protected HashMap<ServiceEndpoint, JmsConnection> endpointConnections;

    protected Logger logger;

    protected ComponentContext context;

    public static final String TYPE = "jms";

    public static final String EXTENSIONS_XMLNS = "http://petals.objectweb.org/extensions";

    public static final String TAG_TYPE = "type";

    protected WsdlFileFilter wsdlFileFilter = new WsdlFileFilter();

    public OutgoingJmsServiceUnitHandler(ComponentContext context, Logger logger) {
        this.logger = logger;
        this.context = context;

        jmsConnexions = new HashMap<String, JmsConnection>();
        suRootPaths = new HashMap<String, String>();
        descriptions = new HashMap<String, Document>();
        endpointConnections = new HashMap<ServiceEndpoint, JmsConnection>();
    }

    /**
     * TODO add customized message
     *  (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#deploy(java.lang.String, java.lang.String, java.lang.String)
     */
    public String deploy(String serviceUnitName, String serviceUnitType,
        String serviceUnitRootPath) throws DeploymentException {

        logger.log(Level.FINE, "deploy");

        String result = null;

        if (jmsConnexions.containsKey(serviceUnitName)) {
            // already bound
            result = ManagementMessageUtil.getComponentTaskResult(context
                .getComponentName(), "deploy",
                ManagementMessageUtil.TASK_RESULT_FAILED);
        } else {
            JmsConnection conn;

            try {
                if (isSupportedTypeServiceUnit(serviceUnitRootPath)) {
                    // supported service unit

                    conn = JmsConnectionFactory.instance().createConnection(
                        serviceUnitRootPath);

                    jmsConnexions.put(serviceUnitName, conn);
                    suRootPaths.put(serviceUnitName, serviceUnitRootPath);

                    Document wsdlDescription = findWSDLDescription(serviceUnitRootPath);

                    descriptions.put(serviceUnitName, wsdlDescription);

                    logger.log(Level.FINE, "deploy  serviceUnitName  "
                        + serviceUnitName + "  serviceUnitRootPath "
                        + serviceUnitRootPath);

                    result = ManagementMessageUtil.getComponentTaskResult(
                        context.getComponentName(), "deploy",
                        ManagementMessageUtil.TASK_RESULT_SUCCESS);
                } else {
                    // not supported service unit, return null
                }
            } catch (Exception e1) {
                logger.log(Level.SEVERE,
                    "Exception occured during deployment.", e1);

                result = ManagementMessageUtil.getComponentTaskResult(context
                    .getComponentName(), "deploy",
                    ManagementMessageUtil.TASK_RESULT_FAILED);
            }
        }
        return result;
    }

    /**
     * Init the JmsConnection object related to the serviceUnitName
     */
    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {

        JmsConnection conn = jmsConnexions.get(serviceUnitName);

        if (conn != null) {
            try {
                conn.init();
            } catch (Exception e) {
                throw new DeploymentException("Can not init serviceUnit '"
                    + serviceUnitName + "'.", e);
            }
        } else {
            throw new DeploymentException("Can not init serviceUnit '"
                + serviceUnitName + "'  : not deployed");
        }

    }

    /**
     * Register the Queue or Topic in JBI with a new ServiceEndpoint,
     * related to the serviceUnitName
     */
    public void start(String serviceUnitName, PetalsServiceUnitManager suManager)
        throws DeploymentException {
        logger.log(Level.FINE, "start");
        logger.log(Level.INFO, "start  serviceUnitName  " + serviceUnitName);

        try {
            //  1. find in the jbi.xml file the 'provides' element 
            //  to retreives the endpoint information
            String suRootPath = suRootPaths.get(serviceUnitName);
            JbiConnection jbiConn = JbiConnectionFactory.instance()
                .createProvidesConnection(suRootPath, logger);

            // 2. activate the endpoint
            ServiceEndpoint ep = context.activateEndpoint(jbiConn
                .getServiceName(), jbiConn.getEndpointName());

            // 3. register in the SU manager the endpoint
            suManager.setServiceDescription(ep, descriptions
                .get(serviceUnitName));
            suManager.setServiceUnitEndpoint(serviceUnitName, ep);

            // 4. associate the jms connection with the endpoint
            JmsConnection jmsConn = jmsConnexions.get(serviceUnitName);
            endpointConnections.put(ep, jmsConn);
        } catch (Exception ex) {
            logger.log(Level.SEVERE, "Activate endpoint failed." + ex);
            throw new DeploymentException("Activate endpoint failed.", ex);
        }
    }

    /**
     * deactivate the endpoint related to the serviceUnitName
     */
    public void stop(String serviceUnitName, PetalsServiceUnitManager suManager)
        throws DeploymentException {
        logger.log(Level.FINE, "stop");
        logger.log(Level.INFO, "stop  serviceUnitName  " + serviceUnitName);

        try {
            // deactivate the endpoint
            ServiceEndpoint ep = suManager
                .getServiceUnitEndpoint(serviceUnitName);
            context.deactivateEndpoint(ep);

            // remove ep->connection association
            endpointConnections.remove(ep);

            suManager.removeServiceDescription(ep);
            suManager.removeServiceUnitEndpoint(serviceUnitName);
        } catch (JBIException e) {
            logger.log(Level.SEVERE, "Deactivate endpoint failed." + e);
            throw new DeploymentException("Deactivate endpoint failed.", e);
        }
    }

    public void shutDown(String serviceUnitName,
        PetalsServiceUnitManager epHandler) throws DeploymentException {
        logger.log(Level.FINE, "shutDown");
        logger.log(Level.INFO, "shutdown  serviceUnitName  " + serviceUnitName);
    }

    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        logger.log(Level.FINE, "undeploy");

        jmsConnexions.remove(serviceUnitName);
        descriptions.remove(serviceUnitName);
        suRootPaths.remove(serviceUnitName);

        return ManagementMessageUtil.getComponentTaskResult(context
            .getComponentName(), "undeploy",
            ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }

    /**
     * Return the JmsConnection associated with the given endpoint
     * @param endpoint
     * @return the connection, null if no association is found
     */
    public JmsConnection getJmsConnection(ServiceEndpoint endpoint) {
        return endpointConnections.get(endpoint);
    }

    /**
     * Checks if this ServiceUnitHandler can deploy this service unit
     * ( in JBI.XML : services/type = jms, 'provides' element not empty )
     * @return true if the service unit type is supported or else false
     */
    protected boolean isSupportedTypeServiceUnit(String serviceUnitRootPath) {

        boolean result = false;

        File jbiXmlFile = new File(serviceUnitRootPath + File.separator
            + "META-INF" + File.separator + "jbi.xml");

        try {

            // 1. check that 'services/provides' element is not empty
            JBIDescriptorBuilder descriptorBuilder = new JBIDescriptorBuilder(
                jbiXmlFile.toURI(), logger);
            JBIDescriptor descriptor = descriptorBuilder.build();
            Services services = descriptor.getServices();

            List providesList = services.getProvides();
            boolean providesOk = (providesList != null && providesList.size() > 0);

            // 2. check that 'services/type' element is set to 'jms'
            Document configDoc = WSDLHelper.createDocumentFromWSDL(jbiXmlFile);

            Node servicesNode = XMLHelper
                .findChild(configDoc, "services", true);

            boolean typeOk = false;

            if (servicesNode != null) {
                Node typeNode = XMLHelper.findChild(servicesNode,
                    EXTENSIONS_XMLNS, TAG_TYPE, false);

                if (typeNode != null
                    && typeNode.getNodeType() == Node.ELEMENT_NODE) {
                    typeOk = (TYPE.equalsIgnoreCase(typeNode.getTextContent()));
                }
            }
            result = providesOk && typeOk;

        } catch (SAXException e) {
            logger.log(Level.SEVERE, "Can not analyse jbi.xml file.", e);
        }

        return result;
    }

    /**
     * Search in the <code>serviceUnitRootPath</code> directory
     * a WSDL file, then create a <code>Document</code> object
     * representing it. 
     * @param serviceUnitRootPath directory where a WSDL file can be found
     * @return a <code>Document</code>
     * @throws Exception WSDL file not found, or can not create a Document from it. 
     */
    protected Document findWSDLDescription(String serviceUnitRootPath)
        throws Exception {
        File[] files = new File(serviceUnitRootPath).listFiles(wsdlFileFilter);

        Document wsdl = null;

        if (files == null || files.length <= 0) {
            throw new FileNotFoundException("No WSDL file was found in '"
                + serviceUnitRootPath + "' directory.");
        }
        wsdl = createDocumentFromWSDL(files[0]);

        if (wsdl == null) {
            throw new Exception(
                "Can not create a Document object from the WSDL file.");
        }
        return wsdl;
    }

    class WsdlFileFilter implements FileFilter {
        public boolean accept(File f) {
            return f.getName().toLowerCase().endsWith(".wsdl");
        }
    }
}
