/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:  JBIExchangeProcessorTest.java 154 2006-03-27 15:30:10Z alouis $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.jms.outgoing;

import java.util.logging.Logger;

import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;

import org.objectweb.petals.component.common.MEPConstants;

import junit.framework.TestCase;
import static org.easymock.classextension.EasyMock.*;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author alouis
 * 
 */
public class JBIExchangeProcessorTest extends TestCase {

    protected Logger log = Logger.getAnonymousLogger();

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /*
     * test that 'connector.sendJMSMessage' is called when all is okay Test
     * method for
     * 'org.objectweb.petals.binding.jms.JBIExchangeProcessor.process(MessageExchange)'
     */
    public void testProcess() throws Exception {
        MessageExchange msgEx = createMock(MessageExchange.class);
        expect(msgEx.getStatus()).andReturn(ExchangeStatus.ACTIVE);
        expect(msgEx.getRole()).andReturn(MessageExchange.Role.PROVIDER);
        expect(msgEx.getPattern()).andReturn(MEPConstants.IN_ONLY_PATTERN);
        msgEx.setStatus(ExchangeStatus.DONE);
        replay(msgEx);

        DeliveryChannel channel = createNiceMock(DeliveryChannel.class);

        JmsConnector jmsConnector = createMock(JmsConnector.class);
        // assume that sendJMSMessage is called;
        jmsConnector.sendJMSMessage(msgEx);
        replay(jmsConnector);

        JBIExchangeProcessor proc = new JBIExchangeProcessor(null, channel,
            null, jmsConnector, log);

        // test
        proc.process(msgEx);
    }

    /*
     * test that 'process' throws an exception when the MsgEx is not an
     * In/RobustIn exchange Test method for
     * 'org.objectweb.petals.binding.jms.JBIExchangeProcessor.process(MessageExchange)'
     */
    public void testProcessFailed1() throws Exception {
        MessageExchange msgEx = createMock(MessageExchange.class);

        expect(msgEx.getStatus()).andReturn(ExchangeStatus.ACTIVE);
        expect(msgEx.getRole()).andReturn(MessageExchange.Role.PROVIDER);
        expect(msgEx.getPattern()).andReturn(MEPConstants.IN_OUT_PATTERN)
            .anyTimes();

        // assume that the exchange will be set as ERROR
        msgEx.setError(isA(Exception.class));
        msgEx.setStatus(ExchangeStatus.ERROR);

        replay(msgEx);

        JBIExchangeProcessor proc = new JBIExchangeProcessor(null, null, null,
            null, log);

        // test
        try {
            proc.process(msgEx);
            fail();
        } catch (Exception e) {
            // 
        }
    }

    /*
     * test that 'process' throws an exception when the jmsConnector failed to
     * send the message Test method for
     * 'org.objectweb.petals.binding.jms.JBIExchangeProcessor.process(MessageExchange)'
     */
    public void testProcessFailed2() throws Exception {
        MessageExchange msgEx = createMock(MessageExchange.class);
        expect(msgEx.getStatus()).andReturn(ExchangeStatus.ACTIVE);
        expect(msgEx.getRole()).andReturn(MessageExchange.Role.PROVIDER);
        expect(msgEx.getPattern()).andReturn(MEPConstants.IN_ONLY_PATTERN);
        msgEx.setStatus(ExchangeStatus.DONE);
        replay(msgEx);

        DeliveryChannel channel = createNiceMock(DeliveryChannel.class);

        JmsConnector jmsConnector = createMock(JmsConnector.class);
        // assume that sendJMSMessage failed;
        jmsConnector.sendJMSMessage(msgEx);
        replay(jmsConnector);

        JBIExchangeProcessor proc = new JBIExchangeProcessor(null, channel,
            null, jmsConnector, log);

        // test
        proc.process(msgEx);
    }

}
