/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:  JmsConnectorTest.java 154 2006-03-27 15:30:10Z alouis $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.jms.outgoing;

import java.lang.reflect.Method;
import java.util.logging.Logger;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.objectweb.petals.binding.jms.JmsConnection;

import junit.framework.TestCase;

import static org.easymock.classextension.EasyMock.*;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author alouis
 * 
 */
public class JmsConnectorTest extends TestCase {

    protected Logger log = Logger.getAnonymousLogger();

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /*
     * Test that 'sendTopicMessage' is called when the related JmsConnection is
     * set to 'topicConnection' Test method for
     * 'org.objectweb.petals.binding.jms.JmsConnector.sendJMSMessage(MessageExchange)'
     */
    public void testSendJMSMessageTopic() throws Exception {
        JmsConnection jmsConn = createMock(JmsConnection.class);
        expect(jmsConn.isTopicConnection()).andReturn(true);
        expect(jmsConn.getDestinationName()).andReturn("destination")
            .anyTimes();
        replay(jmsConn);

        OutgoingJmsServiceUnitHandler suh = createMock(OutgoingJmsServiceUnitHandler.class);
        expect(suh.getJmsConnection(isA(ServiceEndpoint.class))).andReturn(
            jmsConn);
        replay(suh);

        JmsConnector connector = createMock(JmsConnector.class,
            new Method[] {JmsConnector.class.getMethod("sendTopicMessage",
                new Class[] {MessageExchange.class, JmsConnection.class})});
        connector.logger = log;
        connector.jmsSUHandler = suh;

        MessageExchange msgEx = createMock(MessageExchange.class);
        ServiceEndpoint ep = createNiceMock(ServiceEndpoint.class);
        replay(ep);
        expect(msgEx.getEndpoint()).andReturn(ep).anyTimes();
        replay(msgEx);

        // expect that sendTopicMessage will be called
        connector.sendTopicMessage(msgEx, jmsConn);

        replay(connector);

        // test
        connector.sendJMSMessage(msgEx);
    }

    /*
     * Test that 'sendQueueMessage' is called when the related JmsConnection is
     * set to 'queueConnection' Test method for
     * 'org.objectweb.petals.binding.jms.JmsConnector.sendJMSMessage(MessageExchange)'
     */
    public void testSendJMSMessageQueue() throws Exception {
        JmsConnection jmsConn = createMock(JmsConnection.class);
        expect(jmsConn.isTopicConnection()).andReturn(false);
        expect(jmsConn.getDestinationName()).andReturn("destination")
            .anyTimes();
        replay(jmsConn);

        OutgoingJmsServiceUnitHandler suh = createMock(OutgoingJmsServiceUnitHandler.class);
        expect(suh.getJmsConnection(isA(ServiceEndpoint.class))).andReturn(
            jmsConn);
        replay(suh);

        JmsConnector connector = createMock(JmsConnector.class,
            new Method[] {JmsConnector.class.getMethod("sendQueueMessage",
                new Class[] {MessageExchange.class, JmsConnection.class})});
        connector.logger = log;
        connector.jmsSUHandler = suh;

        MessageExchange msgEx = createMock(MessageExchange.class);
        ServiceEndpoint ep = createNiceMock(ServiceEndpoint.class);
        replay(ep);
        expect(msgEx.getEndpoint()).andReturn(ep).anyTimes();
        replay(msgEx);

        // expect that sendTopicMessage will be called
        connector.sendQueueMessage(msgEx, jmsConn);

        replay(connector);

        // test
        connector.sendJMSMessage(msgEx);
    }

    /*
     * Test method for
     * 'org.objectweb.petals.binding.jms.JmsConnector.sendTopicMessage(MessageExchange,
     * JmsConnection)'
     */
    public void testSendTopicMessage() {

    }

    /*
     * Test method for
     * 'org.objectweb.petals.binding.jms.JmsConnector.sendQueueMessage(MessageExchange,
     * JmsConnection)'
     */
    public void testSendQueueMessage() {

    }

    /*
     * Test method for
     * 'org.objectweb.petals.binding.jms.JmsConnector.transform(NormalizedMessage,
     * TextMessage)'
     */
    public void testTransform() {

    }

}
