/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : $${file_name} 154 $${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.smtpbc.listeners;

import java.io.File;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.mail.Address;
import javax.mail.Flags;
import javax.mail.Folder;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.xml.namespace.QName;

import org.ebmwebsourcing.b2bgw.security.beans.CertificatePKCS12;
import org.objectweb.petals.binding.exception.SMTPBCException;
import org.objectweb.petals.component.common.handleMessagesExchange.HandleResponses;
import org.objectweb.petals.component.common.xmlmanipulation.SourceHelper;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.w3c.dom.Document;

/**
 * listener verify perodically a mail store and if there are new messages,
 * binding component get the messages and they are process, and deleted in the
 * end
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class SMTPBCListener implements Runnable {

    static Address daemon;

    protected ComponentContext context;

    protected DeliveryChannel channel;

    protected Folder folder;

    protected String operation;

    protected SenderMail senderMail;

    protected Store store;

    private CertificatePKCS12 bindingCertificate;

    private Properties connectionProps;

    private String host;

    private Logger logger;

    /**
     * Type : simple, signed , signedAndCryted
     */
    private String mailType;

    private boolean running;

    private int wait;

    public SMTPBCListener(Logger logger, CertificatePKCS12 cert,
        Properties props, ComponentContext context, DeliveryChannel channel) {
        super();
        try {
            daemon = new InternetAddress("MAILER-DAEMON@ebmwebsourcing.org");
        } catch (AddressException e) {
            e.printStackTrace();
        }
        this.context = context;
        this.channel = channel;
        this.logger = logger;
        this.connectionProps = props;
        this.bindingCertificate = cert;
        if (props != null) {
            this.host = this.connectionProps.getProperty("host");
            this.mailType = this.connectionProps.getProperty("message.type");
            String str = this.connectionProps.getProperty("thread.wait");

            this.wait = Integer.parseInt(str);
            this.senderMail = new SenderMail(props.getProperty("host"), props
                .getProperty("from"), context.getInstallRoot() + File.separator
                + "certificate" + File.separator, cert, this.logger);
        }
    }

    /**
     * Initialize parameters of IMAP server allowing to connect to it
     * 
     * @throws SMTPBCException
     * 
     */
    public void connectionToHost() throws SMTPBCException {
        try {
            String port = (String) this.connectionProps.get("port");
            String login = (String) this.connectionProps.get("login");
            String password = (String) this.connectionProps.get("password");
            String tranportProtocole = this.connectionProps
                .getProperty("transport.protocole");
            Properties props = new Properties();
            // Get IMAP message store --
            if (tranportProtocole.equals("imap")) {
                // Get default session properties
                props.put("mail.transport.protocol", "pop3");
                Session session = Session.getDefaultInstance(props, null);
                // store = new Store(session, new URLName(host));
                store = session.getStore("imap");
            } else if (tranportProtocole.equals("pop3")) {
                // Get default session properties

                props.put("mail.transport.protocol", "pop3");
                Session session = Session.getDefaultInstance(new Properties(),
                    null);
                store = session.getStore("pop3");
            }

            if (host == null || host.equals("")) {
                throw new SMTPBCException(
                    "You didn't change parameters of connection in connection.properties");
            }
            if (store != null) {
                // Verify if server listen in specific port
                if (port == null || port.equals("")) {
                    store.connect(host, login, password);
                } else {
                    int portInt = Integer.parseInt(port);
                    store.connect(host, portInt, login, password);
                }
            } else {
                throw new SMTPBCException("Error open Store");
            }
        } catch (MessagingException e) {
            this.store = null;
            throw new SMTPBCException("Error open Store", e.getMessage());
        }
    }

    public boolean eraseMail(Message msg) {
        // Set message flag to delete
        try {
            msg.setFlag(Flags.Flag.DELETED, true);
            // Erase all messages tag with Delete flag
            folder.expunge();
            return true;
        } catch (Exception folderException) {
            this.logger.log(Level.WARNING,
                "expunge fonction not supported by this host "
                    + folderException.getMessage());
            return false;
        }
    }

    public String findServiceOperation(String subject) {
        // Take service and operation of subject
        String service = null;
        this.operation = null;
        StringTokenizer st = new StringTokenizer(subject, ";");
        if (st.countTokens() > 0) {
            service = st.nextToken();
            if (st.hasMoreTokens()) {
                this.operation = st.nextToken();
            }
        }
        return service;
    }

    public Set<DataHandler> getAttachments(NormalizedMessage nm) {
        Set<DataHandler> attachments = new HashSet<DataHandler>();
        Set attachHandler = nm.getAttachmentNames();
        for (Object object : attachHandler) {
            String attachName = (String) object;
            attachments.add(nm.getAttachment(attachName));
        }
        return attachments;
    }

    /**
     * Get fault message and send un mail to notify error
     * 
     * @param msg
     * @param to
     * @param subject
     */
    public void handleFault(MessageExchange msg, String to, String subject) {

        NormalizedMessage nm = msg.getFault();
        String fault = null;
        try {
            if (nm != null) {
                fault = SourceHelper.createString(nm.getContent());
                this.senderMail.sendMail(this.mailType, fault, to, "Re:"
                    + subject, getAttachments(nm));

                msg.setStatus(ExchangeStatus.DONE);
                channel.send(msg);
            }
        } catch (Exception e) {
            this.logger.log(Level.WARNING, "Error handle fault "
                + e.getMessage());
        }
    }

    /**
     * Handle inOnly messages
     * 
     * @param mailCorps
     * @param service
     * @param operation
     * @throws SMTPBCException
     */
    public boolean handleInOnlyMessage(String mailCorps, QName service,
        Set<DataHandler> attach) throws SMTPBCException {
        boolean sended = false;
        try {
            InOnly msg = null;
            msg = channel.createExchangeFactory().createInOnlyExchange();
            HandleResponses.buildInMessage(msg, mailCorps, service, operation,
                attach);
            channel.sendSync(msg);
            if (ExchangeStatus.DONE.equals(msg.getStatus())) {
                sended = true;
            }
            return sended;
        } catch (Exception ex) {
            throw new SMTPBCException(ex.getMessage());
        }
    }

    /**
     * Handle inOut Message
     * 
     * @param mailCorps
     * @param service
     * @param operation
     * @param to
     * @param subject
     * @throws SMTPBCException
     */
    public void handleInOutMessage(String mailCorps, QName service, String to,
        String subject, Set<DataHandler> attach) throws SMTPBCException {

        try {

            InOut msg = null;
            msg = channel.createExchangeFactory().createInOutExchange();

            HandleResponses.buildInMessage(msg, mailCorps, service, operation,
                attach);
            channel.sendSync(msg);

            // Maybe it had an error in called service
            if ((ExchangeStatus.ERROR).equals(msg.getStatus())) {
                handleFault(msg, to, subject);
            } else {
                NormalizedMessage nm = msg.getMessage("OUT");
                // Get OUT message
                String response = null;
                if (nm != null) {
                    response = SourceHelper.createString(nm.getContent());

                    // send message to external service
                    this.senderMail.sendMail(mailType, response, to, "Re:"
                        + subject, getAttachments(nm));
                }

            }
            msg.setStatus(ExchangeStatus.DONE);
            channel.send(msg);

        } catch (Exception ex) {
            throw new SMTPBCException(ex.getMessage());
        }
    }

    /**
     * Handle RobustInOnly messages
     * 
     * @param mailCorps
     * @param service
     * @param operation
     * @throws SMTPBCException
     */
    public boolean handleRobustInOnlyMessage(String mailCorps, QName service,
        String to, String subject, Set<DataHandler> attach)
        throws SMTPBCException {
        boolean sended = false;
        try {
            RobustInOnly msg = null;
            msg = channel.createExchangeFactory().createRobustInOnlyExchange();

            HandleResponses.buildInMessage(msg, mailCorps, service, operation,
                attach);
            channel.sendSync(msg);

            ExchangeStatus status = msg.getStatus();

            if (status.equals(ExchangeStatus.ERROR)) {
                // Notify error to external service
                handleFault(msg, to, subject);
                sended = true;
            } else if (status.equals(ExchangeStatus.DONE)) {
                sended = true;
            }
            return sended;
        } catch (Exception ex) {
            throw new SMTPBCException(ex.getMessage());
        }
    }

    /**
     * Takes connection parameters of email compte: login, password, and server
     * mail url
     * 
     * @return
     * @throws SMTPBCException
     */
    public Folder loadFolder() throws SMTPBCException {
        try {
            Folder fold = null;
            // Take parent folder in email count
            fold = store.getDefaultFolder();
            if (fold == null) {
                throw new SMTPBCException("No default folder");
            }
            // Take incoming messages folder
            fold = fold.getFolder("INBOX");
            if (fold == null) {
                throw new SMTPBCException("No IMAP INBOX");
            }

            // Open the folder in read_write to delete the messages that are
            fold.open(Folder.READ_WRITE);
            return fold;
        } catch (MessagingException e) {
            throw new SMTPBCException("Error open folder", e.getMessage());
        }
    }

    /**
     * Handle each message
     * 
     * @param msg
     * @throws SMTPBCException
     */
    public void manageMessage(Message msg) {

        String service = null;
        boolean inOut = false;
        boolean erase = false;
        QName qService = null;
        Set<DataHandler> attachments = new HashSet<DataHandler>();
        try {
            String mailCorps = MessageHandler.handleMessage(msg,
                this.bindingCertificate, attachments);

            service = findServiceOperation(msg.getSubject());
            if (service != null) {
                qService = QName.valueOf(service);
            }
            ServiceEndpoint[] endpoints = context
                .getEndpointsForService(qService);

            Address[] from = msg.getFrom();
            if (daemon.equals(from[0])) {
                erase = true;
            } else {
                if (endpoints.length > 0) {
                    if (mailCorps != null || !attachments.isEmpty()) {
                        inOut = isInOut(endpoints[0]);
                        // If is a inOut operation this method handle message
                        if (inOut) {
                            handleInOutMessage(mailCorps, qService, from[0]
                                .toString(), msg.getSubject(), attachments);
                            erase = true;
                        } else {
                            // if operation is a inOut Method
                            erase = handleInOnlyMessage(mailCorps, qService,
                                attachments);
                        }
                    }
                } else {
                    serviceNotfound(from[0].toString(), service);
                    erase = true;
                }
            }
        } catch (Exception ex) {
            this.logger.log(Level.WARNING, "Error handle message "
                + ex.getMessage());
            erase = true;
        }
        if (erase) {
            eraseMail(msg);
        }
    }

    /**
     * Get all new messages call service ,specified in subject of mail, if
     * service doesn't exist send mail to sender with original mail attached
     * 
     * @throws SMTPBCException
     */
    public void process() {
        Message[] msgs = {};
        try {
            if (folder != null) {
                msgs = folder.getMessages();
            } else {
                this.logger.log(Level.WARNING, "folder isn't initialized ");
            }
        } catch (MessagingException e) {
            this.logger.log(Level.WARNING, "Error getting messages"
                + e.getMessage());
        }
        for (Message message : msgs) {
            manageMessage(message);
        }
    }

    public void run() {

        running = true;
        try {
            connectionToHost();
            this.folder = loadFolder();

            while (running) {
                try {
                    process();
                    Thread.sleep(wait);
                } catch (Exception ex) {
                    logger.log(Level.SEVERE, ex.getMessage(), ex);
                }
            }
        } catch (SMTPBCException e) {
            this.logger.log(Level.SEVERE,
                "Error getting parameters of connection");
        }
    }

    /**
     * close folder and store when the binding component is stopped
     * 
     * @throws SMTPBCException
     */
    public void stopProcess() throws SMTPBCException {

        try {
            running = false;
            if ((folder != null) && (folder.isOpen())) {
                folder.close(false);
            }
            if ((store != null) && (store.isConnected())) {
                store.close();
            }
        } catch (Exception ex) {
            this.folder = null;
            this.store = null;
            throw new SMTPBCException(" Error close folder or IMAP Store", ex
                .getMessage());
        }
    }

    protected boolean isInOut(ServiceEndpoint endpoint) throws SMTPBCException {
        boolean found = false;
        boolean inOut = false;
        Document serviceDesc;
        try {
            serviceDesc = context.getEndpointDescriptor(endpoint);
            QName service = null;
            try {
                service = WSDLHelper
                    .getServiceNameFromWSDLDocument(serviceDesc)[0];
            } catch (Exception e1) {
                e1.printStackTrace();
            }
            // Determines if the operation exists and if it is an inOnly or
            // inOut operation
            if (this.operation != null) {
                try {
                    found = WSDLHelper.hasOperationNamed(serviceDesc,
                        this.operation, service);
                    inOut = WSDLHelper.isInOutOperation(serviceDesc,
                        this.operation, service);
                } catch (Exception e) {
                    throw new SMTPBCException(e.getMessage());
                }

            } else if (!found) {
                // if operation was not specified
                inOut = true;
            }
        } catch (JBIException e) {
            this.logger.log(Level.WARNING, "fileDescriptor of endpoint "
                + endpoint.getEndpointName() + "not found");
        }
        return inOut;
    }

    protected void serviceNotfound(String from, String service) {
        // There is an unique sender, so from has only one address
        logger.log(Level.INFO, "SMTPBC : Service " + service
            + "  doesn't exist to " + from);
        String fault = SourceHelper.createSoapFault(new SMTPBCException(
            "Service unknown"), "300");
        try {
            this.senderMail.sendMail("simple", fault, from,
                "service not available", null);
        } catch (SMTPBCException ex) {
            this.logger.log(Level.WARNING, ex.getMessage());
        }
    }

}
