/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.binding.smtpbc;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.io.File;

import javax.jbi.JBIException;
import javax.jbi.component.InstallationContext;

import junit.framework.TestCase;

import org.objectweb.petals.binding.smtpbc.SMTPBCBootstrapImpl;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 * 
 */
public class SMTPBCBoostrapTest extends TestCase {
    File installationRoot = null;

    public void testCleanUp() {

        SMTPBCBootstrapImpl SMTPbootstrap = new SMTPBCBootstrapImpl();

        // Run the Test
        try {
            SMTPbootstrap.cleanUp();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in SMTPBCBootstrap clean up");
        }
    }

    public void testGetExtensionMBeanName() {
        // Create objects
        SMTPBCBootstrapImpl SMTPbootstrap = new SMTPBCBootstrapImpl();

        // Run the Test
        Object result = null;
        result = SMTPbootstrap.getExtensionMBeanName();
        assertNull("Result of invocation should be null", result);
    }

    public void testInit() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
                installationRoot.getAbsolutePath());
        replay(installationContext);

        SMTPBCBootstrapImpl smtpbootstrap = new SMTPBCBootstrapImpl();
        // Run the Test
        try {
            smtpbootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in SMTPBCBootstrap init method");
        }

        assertEquals(smtpbootstrap.context, installationContext);
    }

    public void testOnInstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
                installationRoot.getAbsolutePath());
        replay(installationContext);

        // Create test parameters
        File metaInfDir = new File(installationRoot.getAbsolutePath()
                + File.separator + "META-INF");
        metaInfDir.mkdirs();
        File caCert = new File(metaInfDir.getAbsolutePath() + File.separator
                + "ca.crt");
        File compCert = new File(metaInfDir.getAbsolutePath() + File.separator
                + "smtpbc.p12");
        try {
            caCert.createNewFile();
            compCert.createNewFile();
        } catch (Exception e) {
            e.printStackTrace();
        }

        SMTPBCBootstrapImpl SMTPbootstrap = new SMTPBCBootstrapImpl();
        // Run the Test
        try {
            SMTPbootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in SMTPBCBootstrap onInstall method");
        }

        // Run the Test
        try {
            SMTPbootstrap.onInstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in SMTPBCBootstrap onInstall method");
        }
        verify(installationContext);
        File[] installDirfiles = installationRoot.listFiles();
        boolean certDirCreated = false;
        for (File file : installDirfiles) {
            if (file.getName().equals("certificate")) {
                certDirCreated = true;
            }
        }
        assertTrue(certDirCreated);
    }

    public void testOnUninstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
                installationRoot.getAbsolutePath()).anyTimes();
        replay(installationContext);

        SMTPBCBootstrapImpl SMTPbootstrap = new SMTPBCBootstrapImpl();
        // Run the Test
        try {
            SMTPbootstrap.init(installationContext);
            SMTPbootstrap.onInstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in SMTPBCBootstrap init or onUninstall method");
        }

        // Run the Test
        try {
            SMTPbootstrap.onUninstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in SMTPBCBootstrap onuninstall method");
        }
        verify(installationContext);
        File certDir = new File(installationRoot.getAbsolutePath()
                + File.separator + "certificate");
        assertEquals(0, certDir.length());
    }

    @Override
    protected void setUp() throws Exception {
        File file = new File(".");
        installationRoot = new File(file.getAbsolutePath() + File.separator
                + "src" + File.separator + "test" + File.separator + "files"
                + File.separator + "bootstrap");
        installationRoot.mkdirs();

    }
}
