/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:  MessageHandlerTest.java 154 2006-03-27 15:30:10Z rbarraza $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.smtpbc.listeners;

import java.io.File;
import java.io.FileInputStream;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Properties;

import javax.mail.Message;
import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import junit.framework.TestCase;

import org.ebmwebsourcing.b2bgw.security.SecurityManager;
import org.ebmwebsourcing.b2bgw.security.beans.CertificatePKCS12;
import org.objectweb.petals.binding.smtpbc.listeners.MessageHandler;
import org.objectweb.petals.binding.smtpbc.listeners.MessageMime;

public class MessageHandlerTest extends TestCase {

    private String baseDir;

    public Message conf() {
        Properties props = System.getProperties();

        String from = "sender@test.com";
        String host = "host";
        props.put("mail.smtp.user", from);
        props.put("mail.smtp.host", host);

        Session session = Session.getDefaultInstance(props, null);
        session.setDebug(false);
        Message message = new MimeMessage(session);
        try {
            /**
             * Set all recipients to send message
             */
            message.setFrom(new InternetAddress(from));

            message.addRecipient(javax.mail.Message.RecipientType.TO,
                new InternetAddress("dest@test.com"));

            message.setSubject("externalService");
        } catch (Exception e) {
            fail("Error in initializing properties");
        }
        return message;
    }

    @Override
    public void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        super.setUp();
    }

    public void testSignedAndCryptedMessage() {

        // Create certificate factory to load X509 certificates.
        CertificateFactory cf;
        try {
            cf = CertificateFactory.getInstance("X.509");

            // load autority certificate
            X509Certificate caCert = (X509Certificate) cf
                .generateCertificate(new FileInputStream(baseDir + "src"
                    + File.separator + "conf" + File.separator + "ca.crt"));

            // load receiver certificate, to get public key
            X509Certificate receiptCert = (X509Certificate) cf
                .generateCertificate(new FileInputStream(baseDir + "src"
                    + File.separator + "conf" + File.separator + "smtpbc.csr"));

            // load sender certificate
            CertificatePKCS12 testCert = SecurityManager.loadPKCS12Certificate(
                new FileInputStream(baseDir + "src" + File.separator + "conf"
                    + File.separator + "smtpbc.p12"), "password");
            // Create signed and cryted Mime message, first signed , after
            // encryted message
            Message message = MessageMime.signedAndEnvelope((MimeMessage) this
                .conf(), testCert, caCert, receiptCert, "crypted body", null);

            MessageHandler.handleMessage(message, testCert, null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error in initializing properties");
        }
    }

    public void testSignedMessage() {

        // Create certificate factory to load X509 certificates.
        CertificateFactory cf;
        try {
            cf = CertificateFactory.getInstance("X.509");

            // load autority certificate
            X509Certificate caCert = (X509Certificate) cf
                .generateCertificate(new FileInputStream(baseDir + "src"
                    + File.separator + "conf" + File.separator + "ca.crt"));
            // load sender certificate

            CertificatePKCS12 testCert = SecurityManager.loadPKCS12Certificate(
                new FileInputStream(baseDir + "src" + File.separator + "conf"
                    + File.separator + "smtpbc.p12"), "password");

            Message msg = MessageMime.signedMultipart(
                (MimeMessage) this.conf(), testCert, caCert, "signed body",
                null);
            MessageHandler.handleMessage(msg, null, null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Error handle signedMultipart message");
        }

    }

    public void testSimpleMessage() {

        Message msg = MessageMime.simpleMessage((MimeMessage) this.conf(),
            "simple body", null);
        try {
            MessageHandler.handleMessage(msg, null, null);
        } catch (Exception e) {
            fail("Error in simple message");
        }
    }
}
