/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.xslt;

import java.io.File;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.management.DeploymentException;
import javax.xml.namespace.QName;

import org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler;
import org.objectweb.petals.component.common.serviceunitmanager.manager.PetalsServiceUnitManager;
import org.objectweb.petals.component.common.xmlmanipulation.ManagementMessageUtil;
import org.objectweb.petals.component.common.xmlmanipulation.WSDLHelper;
import org.w3c.dom.Document;

/**
 * 
 * @version $Rev: 250 $Date: {date}
 * @since   Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 *
 */
public class XsltSUHandler extends
PetalsServiceUnitHandler {

    protected Map<String, String> mapServiceXslt;
    
    protected Map<String, String> mapServiceUnitXslt;


    private XsltSUHandler() {
        super();
        SERVICE_UNIT_TYPE="xsltServiceUnit";
    }

    public XsltSUHandler(ComponentContext context, Map<String, String> mapServiceXslt, Logger logger) {
        super(context, logger);
        this.mapServiceXslt= mapServiceXslt;
        this.mapServiceUnitXslt = new HashMap<String, String>();
        SERVICE_UNIT_TYPE="xsltServiceUnit";
    }

    /**
     *  (non-Javadoc)
     * @see org.objectweb.petals.component.common.serviceunitmanager.handler.PetalsServiceUnitHandler#deploy(java.lang.String, java.lang.String, java.lang.String)
     */
    public String deploy(String serviceUnitName, String serviceUnitType
            , String serviceUnitRootPath) {
        if (SERVICE_UNIT_TYPE.equals(serviceUnitType)) {
            System.out.println(" DEPLOY  serviceUnitName  " + serviceUnitName
                    + "  serviceUnitRootPath " + serviceUnitRootPath);
            return ManagementMessageUtil.getComponentTaskResult(context
                    .getComponentName(), "deploy",
                    ManagementMessageUtil.TASK_RESULT_SUCCESS);
        } else {
            return null;
        }
    }

    public void init(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        this.serviceUnitInstallationRootPath.put(serviceUnitName, serviceUnitRootPath);
    }

   
    public String undeploy(String serviceUnitName, String serviceUnitRootPath)
        throws DeploymentException {
        return ManagementMessageUtil.getComponentTaskResult(context
                .getComponentName(), "undeploy",
                ManagementMessageUtil.TASK_RESULT_SUCCESS);
    }


    public void shutDown(String serviceUnitName, PetalsServiceUnitManager epHandler) throws DeploymentException {
        String xslt = this.mapServiceUnitXslt.remove(serviceUnitName);
        this.mapServiceXslt.remove(xslt);
    }

    public void start(String serviceUnitName, PetalsServiceUnitManager epHandler) throws DeploymentException {
        logger.log(Level.FINE, "start  serviceUnitName  " + serviceUnitName);
        
    //    File installPath = new File(context.getInstallRoot());
        String serviceUnitRootPath = serviceUnitInstallationRootPath.get(serviceUnitName);
        File[] files = new File(serviceUnitRootPath).listFiles();
        Document serviceDesc = null;
        String fileName=null;
        QName service=null;
        
        for (File file : files) {
            if (file.getName().endsWith(".wsdl")) {
                serviceDesc = WSDLHelper.createDocumentFromWSDL(file);
            }
            else if(file.getName().endsWith(".xsl")){
              //  File xslt = new File(installPath, "xsltSheets");
                
                //xslt.mkdirs();
                //file.renameTo(new File(xslt , file.getName()));
                //fileName= xslt.getAbsolutePath()+File.separator+file.getName();
                fileName= file.getAbsolutePath();
            }
        }
        if (serviceDesc != null) {
            try {
                activateEndpointsFromJBIDescription(epHandler, serviceDesc,
                        serviceUnitName,serviceUnitRootPath);
                service=WSDLHelper.getServiceNameFromWSDLDocument(serviceDesc)[0];
            } catch (Exception ex) {
                logger.log(Level.SEVERE, FAILED_ACTIVATE_ENDPOINT + ex);
                throw new DeploymentException(FAILED_ACTIVATE_ENDPOINT, ex);
            }
        } else {
            throw new DeploymentException(INCOMPLETE_SERVICE_UNIT_PACKAGE);
        }
        if(service!=null && fileName != null){
            mapServiceUnitXslt.put(serviceUnitName, service.getLocalPart());
            mapServiceXslt.put( service.getLocalPart(),fileName);
        }
    }

    public void stop(String serviceUnitName, PetalsServiceUnitManager epHandler) throws DeploymentException {
        /*
         * When service unit is stopped, so properties of directory are deleted
         */
        String xslt = this.mapServiceUnitXslt.remove(serviceUnitName);
        this.mapServiceXslt.remove(xslt);
    }
}
