/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza 
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.xslt;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.verify;

import java.io.File;

import javax.jbi.JBIException;
import javax.jbi.component.InstallationContext;

import junit.framework.TestCase;

import org.objectweb.petals.engine.xslt.XsltBootstrapImpl;

/**
 * @version	$Rev: 250 $Date: ${date}
 * @since	Petals 1.0
 * @author Rocio BARRAZA - eBMWebsourcing
 *
 */
public class XsltBootstrapImplTest extends TestCase{

    File installationRoot = null;

    private String baseDir;

    public void testCleanUp() {

        XsltBootstrapImpl xsltbootstrap = new XsltBootstrapImpl();

        // Run the Test
        try {
            xsltbootstrap.cleanUp();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in xsltBCBootstrap clean up");
        }
    }

    public void testGetExtensionMBeanName() {
        // Create objects
        XsltBootstrapImpl xsltbootstrap = new XsltBootstrapImpl();

        // Run the Test
        Object result = null;
        result = xsltbootstrap.getExtensionMBeanName();
        assertNull("Result of invocation should be null", result);
    }

    public void testInit() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
            installationRoot.getAbsolutePath());
        replay(installationContext);

        XsltBootstrapImpl xsltbootstrap = new XsltBootstrapImpl();
        // Run the Test
        try {
            xsltbootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XsltBootstrap init method");
        }

        assertEquals(xsltbootstrap.installContext , installationContext);
    }

    public void testOnInstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        XsltBootstrapImpl xsltbootstrap = new XsltBootstrapImpl();
        // Run the Test
        try {
            xsltbootstrap.init(installationContext);
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XsltBootstrap onInstall method");
        }
    }

    public void testOnUninstall() {
        // Create Mock Objects
        InstallationContext installationContext = createMock(InstallationContext.class);
        // Init mocks
        expect(installationContext.getInstallRoot()).andReturn(
            installationRoot.getAbsolutePath()).anyTimes();
        expect(installationContext.getInstallationDescriptorExtension()).andReturn(
                null);
        replay(installationContext);

        XsltBootstrapImpl xsltbootstrap = new XsltBootstrapImpl();
        // Run the Test
        try {
            xsltbootstrap.init(installationContext);
            xsltbootstrap.onInstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XsltBootstrap init or onUninstall method");
        }

        // Run the Test
        try {
            xsltbootstrap.onUninstall();
        } catch (JBIException e) {
            e.printStackTrace();
            fail("Error in XsltBootstrap onuninstall method");
        }
        verify(installationContext);
    }

    @Override
    protected void setUp() throws Exception {
        baseDir = this.getClass().getResource(".").toString();
        baseDir = baseDir.substring(0, baseDir.indexOf("target"));
        baseDir = baseDir.substring(baseDir.indexOf(":") + 1);
        installationRoot = new File(baseDir + File.separator + "src"
            + File.separator + "test-data" + File.separator + "bootstrap");

    }

}


