/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ShutdownHook.java 1:04:18 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.system.terminate;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.util.Fractal;

/**
 * Hook used to stop Petals
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class ShutdownHook extends Thread {

    private LifeCycleController lifeCycleController;

    private ContentController contentController;

    public ShutdownHook(LifeCycleController lifeCycleController,
            ContentController contentController) {
        super();
        this.lifeCycleController = lifeCycleController;
        this.contentController = contentController;
    }

    public void run() {
        try {
            if (lifeCycleController.getFcState().equals(
                    LifeCycleController.STARTED)) {

                // Stop JoramTransporter component first
                stopJoramTransporter();
                
                // Stop Installation Service
                stopInstallationService();
                
                // Stop the others composites
                lifeCycleController.stopFc();

                System.out.println("Petals is "
                        + lifeCycleController.getFcState());
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
            System.err.println("Problem while stopping Petals");
        }
    }

    protected void stopJoramTransporter() throws NoSuchInterfaceException,
        IllegalLifeCycleException {
        Component jbi = getComponentByName(contentController, "jbi");
        ContentController jbiController = Fractal.getContentController(jbi);
        Component transporter = getComponentByName(jbiController,
                "transporter-impl");
        LifeCycleController transporterLifeCycleController = Fractal
                .getLifeCycleController(transporter);

        transporterLifeCycleController.stopFc();
    }
    
    protected void stopInstallationService() throws NoSuchInterfaceException,
    IllegalLifeCycleException {
    Component jbi = getComponentByName(contentController, "jbi");
    ContentController jbiController = Fractal.getContentController(jbi);
    Component installationServ = getComponentByName(jbiController,
            "installation-impl");
    LifeCycleController installationServLifeCycleController = Fractal
            .getLifeCycleController(installationServ);

    installationServLifeCycleController.stopFc();
}

    /**
     * A utility function allowing to get a component from a content controller
     * 
     * @param cc
     *            ContentController
     * @param name
     *            component name
     * @return
     */
    protected Component getComponentByName(ContentController contentController,
            String name) {
        // The component to be returned
        Component c = null;
        // List content controller subcomponents
        Component[] comps = contentController.getFcSubComponents();
        for (int i = 0; i < comps.length; i++) {
            Component component = comps[i];
            try {
                String componentName = Fractal.getNameController(component)
                        .getFcName();
                if (componentName.equals(name)) {
                    c = component;
                    break;
                }
            } catch (NoSuchInterfaceException e) {
                // Do nothing
            }
        }
        return c;
    }

}
