/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SADeploymentManagerTest.java 154 18 aot 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.service;

import java.io.File;
import java.lang.reflect.Method;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.ZipFile;

import javax.jbi.JBIException;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.xml.namespace.QName;

import junit.framework.TestCase;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.management.autoload.AutoLoaderImpl;
import org.objectweb.petals.jbi.management.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.service.util.XMLResult;
import org.objectweb.petals.jbi.management.service.util.XMLResult.CauseFramework;
import org.objectweb.petals.jbi.management.service.util.XMLResult.MessageType;
import org.objectweb.petals.jbi.management.service.util.XMLResult.TaskResult;
import org.objectweb.petals.repository.RepositoryImpl;
import org.objectweb.petals.repository.RepositoryService;
import org.objectweb.petals.repository.RepositoryImpl.EntityType;
import org.objectweb.petals.tools.jbicommon.descriptor.Connection;
import org.objectweb.petals.tools.jbicommon.descriptor.Identification;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceAssembly;
import org.objectweb.petals.tools.jbicommon.descriptor.ServiceUnit;
import org.objectweb.petals.util.LoggingUtil;

import static org.easymock.EasyMock.aryEq;
import static org.easymock.EasyMock.contains;
import static org.easymock.EasyMock.eq;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;

/**
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SADeploymentManagerTest extends TestCase {

    @Override
    protected void setUp() {

    }

    public void testExpandSUIntoSA1() {
        /*
         * Test Case 1 the explodeSUIntoSAInstallFolder mthod throw an exception
         */
        SADeploymentManager deploymentManager = new SADeploymentManager();
        /*
         * Create Test parameters
         */
        String suId = "serviceUnitId";
        String saId = "serviceAssemblyId";
        URI suZipLocation = null;
        try {
            suZipLocation = new URI("file://zipLocation");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentManager.log = log;
        RepositoryService repoSrv = createMock(RepositoryImpl.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(
                    repoSrv.explodeSUIntoSAInstallFolder(suId, suZipLocation,
                            saId)).andThrow(
                    new PetalsException("explodeSUIntoSA"));
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        deploymentManager.repositorySrv = repoSrv;
        /*
         * Run Test Case 1
         */
        try {
            deploymentManager.expandSUIntoSA(suId, suZipLocation, saId);
            fail("expandSUIntoSA must throw an exception if RepositoryService.explodeSUIntoSAInstallFolder throws an exception");
        } catch (ManagementException e) {
            // Do nothing it's ok
        }

    }

    public void testExpandSUIntoSA2() {
        /*
         * Test Case 2 the result of explodeSUIntoSAInstallFolder is null
         */
        SADeploymentManager deploymentManager = new SADeploymentManager();
        /*
         * Create Test parameters
         */
        String suId = "serviceUnitId";
        String saId = "serviceAssemblyId";
        URI suZipLocation = null;
        try {
            suZipLocation = new URI("file://zipLocation");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentManager.log = log;
        RepositoryService repoSrv = createMock(RepositoryImpl.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(
                    repoSrv.explodeSUIntoSAInstallFolder(suId, suZipLocation,
                            saId)).andReturn(null);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        deploymentManager.repositorySrv = repoSrv;
        /*
         * Run Test Case 2
         */
        try {
            deploymentManager.expandSUIntoSA(suId, suZipLocation, saId);
            fail("expandSUIntoSA must throw an exception if result of explodeSUIntoSAInstallFolder is null");
        } catch (ManagementException e) {
            // Do nothing it's ok
        }

    }

    public void testExpandSUIntoSA3() {
        /*
         * Test Case 3 the result of explodeSUIntoSAInstallFolder is a rootURI
         */
        SADeploymentManager deploymentManager = new SADeploymentManager();
        /*
         * Create Test parameters
         */
        String suId = "serviceUnitId";
        String saId = "serviceAssemblyId";
        URI suZipLocation = null;
        URI suRootLocation = null;
        try {
            suZipLocation = new URI("file://zipLocation");
            suRootLocation = new URI("file://suRootLocation");
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not create test parameters");
        }
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentManager.log = log;
        RepositoryService repoSrv = createMock(RepositoryImpl.class);
        /*
         * Initialize Mocks
         */
        try {
            expect(
                    repoSrv.explodeSUIntoSAInstallFolder(suId, suZipLocation,
                            saId)).andReturn(suRootLocation);
        } catch (Exception e) {
            e.printStackTrace();
            fail("Could not initialize Mocks");
        }
        replay(repoSrv);
        deploymentManager.repositorySrv = repoSrv;
        /*
         * Run Test Case 3
         */
        URI result = null;
        try {
            result = deploymentManager
                    .expandSUIntoSA(suId, suZipLocation, saId);
        } catch (ManagementException e) {
            e.printStackTrace();
            fail("unexpected error when expending su into sa install folder");
        }
        assertEquals("Wrong URI returned", result, suRootLocation);

    }

    public void testGetServiceAssemblyName() {
        /*
         * Test Case 1
         */
        SADeploymentManager deploymentManager = new SADeploymentManager();
        /*
         * Create Test parameters
         */
        String saName = "serviceAssemblyName";
        ServiceAssembly sa = createMock(ServiceAssembly.class);
        Identification id = createMock(Identification.class);
        deploymentManager.assembly = sa;
        /*
         * Initialize Mocks
         */
        expect(sa.getIdentification()).andReturn(id);
        expect(id.getName()).andReturn(saName);
        replay(sa);
        replay(id);
        /*
         * Run Test Case 1
         */
        assertEquals("Wrong invokation result", saName, deploymentManager
                .getServiceAssemblyName());
        verify(sa);
        verify(id);
    }

    public void testRegisterConnection() {
        /*
         * Test Case 1 Cons interface of the connection is not null
         */
        // Create Mock Objects
        Method method1 = null;
        try {
            method1 = SADeploymentManager.class.getDeclaredMethod(
                    "getServiceAssemblyName", new Class[] {});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};

        SADeploymentManager deploymentManager = createMock(
                SADeploymentManager.class, mockedMethods);
        /*
         * Create Test parameters
         */
        Connection connection = createMock(Connection.class);
        EndpointService endpointService = createMock(EndpointService.class);
        String saName = "serviceAssemblyName";
        String provEP = "foo";
        QName provServ = new QName("foo2", "bar2");
        QName consServ = new QName("foo3", "bar3");
        String consEP = "foo4";
        QName consInt = new QName("foo5", "bar5");

        /*
         * Initialize Mocks
         */
        expect(deploymentManager.getServiceAssemblyName()).andReturn(saName);
        expect(connection.getProviderEndpointName()).andReturn(provEP);
        expect(connection.getProviderServiceName()).andReturn(provServ);
        expect(connection.getConsumerServiceName()).andReturn(consServ);
        expect(connection.getConsumerEndpointName()).andReturn(consEP);
        expect(connection.getConsumerInterfaceName()).andReturn(consInt);
        try {
            endpointService.createConnection(consInt, provServ, provEP);
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(connection);
        replay(endpointService);
        deploymentManager.endpointService = endpointService;
        /*
         * Run Test Case 1
         */
        try {
            deploymentManager.registerConnection(connection);
        } catch (ManagementException e) {
            e.printStackTrace();
            fail("Unexpected error durring register connection");
        }
        verify(endpointService);

        /*
         * Test Case 2 Cons interface of the connection is null
         */
        reset(deploymentManager);
        reset(connection);
        reset(endpointService);
        /*
         * Create Test parameters
         */
        /*
         * Initialize Mocks
         */
        expect(deploymentManager.getServiceAssemblyName()).andReturn(saName);
        expect(connection.getProviderEndpointName()).andReturn(provEP);
        expect(connection.getProviderServiceName()).andReturn(provServ);
        expect(connection.getConsumerServiceName()).andReturn(consServ);
        expect(connection.getConsumerEndpointName()).andReturn(consEP);
        expect(connection.getConsumerInterfaceName()).andReturn(null);
        try {
            endpointService
                    .createConnection(consServ, consEP, provServ, provEP);
        } catch (JBIException e) {
            e.printStackTrace();
        }
        replay(connection);
        replay(endpointService);
        deploymentManager.endpointService = endpointService;
        /*
         * Run Test Case 2
         */
        try {
            deploymentManager.registerConnection(connection);
        } catch (ManagementException e) {
            e.printStackTrace();
            fail("Unexpected error durring register connection");
        }
        verify(endpointService);
    }

    public void testCheckConnections() {
        // Create Mock and init param
        Connection connection = createMock(Connection.class);
        SADeploymentManager deploymentManager = new SADeploymentManager();
        List<Connection> connections = new ArrayList<Connection>();

        /*
         * Test Case 1 : consumer interface and (consumer endpoint or consumer
         * service) are null at the same time
         */
        // init mock
        expect(connection.getConsumerInterfaceName()).andReturn(null);
        expect(connection.getConsumerEndpointName()).andReturn(null);
        expect(connection.getConsumerServiceName()).andReturn(null);
        expect(connection.getProviderEndpointName()).andReturn("foo");
        expect(connection.getProviderServiceName()).andReturn(new QName("bar"));
        replay(connection);
        connections.add(connection);

        // Run Test Case 1
        try {
            deploymentManager.checkConnections(connections);
            fail("Must throw an exception if consumer interface and "
                    + "(consumer endpoint or consumer service) are null at the same time");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }

        /*
         * Test Case 2 : provider endpoint and provider service are null or
         * empty
         */
        reset(connection);
        // init mock
        expect(connection.getConsumerInterfaceName()).andReturn(
                new QName("foo"));
        expect(connection.getConsumerEndpointName()).andReturn("bar");
        expect(connection.getConsumerServiceName()).andReturn(new QName("bar"));
        expect(connection.getProviderEndpointName()).andReturn(null);
        expect(connection.getProviderServiceName()).andReturn(null);
        replay(connection);
        connections.clear();
        connections.add(connection);

        // Run Test Case 2
        try {
            deploymentManager.checkConnections(connections);
            fail("Must throw an exception if provider endpoint "
                    + "and provider service are null or empty");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }
    }

    public void testCheckSUIdentifier() {
        // Create init param
        SADeploymentManager deploymentManager = new SADeploymentManager();
        List<String> suIds = new ArrayList<String>();
        String suName = "foo";

        /*
         * Test Case 1 : su identifier is null
         */

        // Run Test Case 1
        try {
            deploymentManager.checkSUIdentifier(suIds, null);
            fail("Must throw an exception if su identifier is null");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }

        /*
         * Test Case 2 : su identifier is non unique within the sa
         */
        // init param
        suIds.add(suName);

        // Run Test Case 2
        try {
            deploymentManager.checkSUIdentifier(suIds, suName);
            fail("Must throw an exception if su identifier non unique within the sa");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }
    }

    public void testCheckTargetComponentAndZip() {
        // Create init param
        SADeploymentManager deploymentManager = new SADeploymentManager();
        String suIdentifier = "foo";
        String artifactZip = "foo.zip";
        String targetComponent = "bar";

        /*
         * Test Case 1 : artefact zip is null
         */

        // Run Test Case 1
        try {
            deploymentManager.checkTargetComponentAndZip(null, suIdentifier,
                    targetComponent);
            fail("Must throw an exception if artefact zip is null");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }

        /*
         * Test Case 2 : target component is null
         */

        // Run Test Case 2
        try {
            deploymentManager.checkTargetComponentAndZip(artifactZip,
                    suIdentifier, null);
            fail("Must throw an exception if target component is null");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }
    }

    public void testCheckSUZipInSAZip() {
        // Create Mock Objects and init parameters
        Method method1 = null;
        Method method2 = null;
        try {
            method1 = SADeploymentManager.class.getDeclaredMethod("hasEntry",
                    new Class[] {ZipFile.class, String.class});
            method2 = SADeploymentManager.class.getDeclaredMethod(
                    "openZipFile", new Class[] {URI.class});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1, method2};

        SADeploymentManager deploymentManager = createMock(
                SADeploymentManager.class, mockedMethods);
        ZipFile zipArchive = createMock(ZipFile.class);
        String artifactZip = "foo.zip";
        URI saArchiveURI = null;
        try {
            saArchiveURI = new URI("file:/bar.zip");
        } catch (URISyntaxException e1) {
            e1.printStackTrace();
        }

        /*
         * Test Case 1 : artefact zip is missing
         */
        // init mocks
        try {
            expect(deploymentManager.openZipFile(saArchiveURI)).andReturn(
                    zipArchive);
        } catch (PetalsException e1) {
            e1.printStackTrace();
        }
        expect(deploymentManager.hasEntry(zipArchive, artifactZip)).andReturn(
                false);

        // Run Test Case 1
        try {
            deploymentManager.checkSUZipInSAZip(artifactZip);
            fail("Must throw an exception if artefact zip is missing");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }

        /*
         * Test Case 2 : zip file can't be opened
         */
        reset(deploymentManager);
        // init mocks
        try {
            expect(deploymentManager.openZipFile(saArchiveURI)).andThrow(
                    new PetalsException(""));
        } catch (PetalsException e1) {
            e1.printStackTrace();
        }

        // Run Test Case 2
        try {
            deploymentManager.checkSUZipInSAZip(artifactZip);
            fail("Must throw an exception if artefact zip can't be opened");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }
    }

    @SuppressWarnings("unchecked")
    public void testCheckServiceUnits() {
        // Create Mock Objects and init parameters
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        try {
            method1 = SADeploymentManager.class
                    .getDeclaredMethod("checkSUIdentifier", new Class[] {
                        List.class, String.class});
            method2 = SADeploymentManager.class.getDeclaredMethod(
                    "checkTargetComponentAndZip", new Class[] {String.class,
                        String.class, String.class});
            method3 = SADeploymentManager.class.getDeclaredMethod(
                    "checkSUZipInSAZip", new Class[] {String.class});
            method4 = SADeploymentManager.class.getDeclaredMethod(
                    "isSUAlreadyDeployedOnTargetedComponent",
                    new Class[] {ServiceUnit.class});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4};

        SADeploymentManager deploymentManager = createMock(
                SADeploymentManager.class, mockedMethods);
        ServiceUnit su = createMock(ServiceUnit.class);
        Identification identification = createMock(Identification.class);
        String suIdentifier = "foo";
        String artifactZip = "foo.zip";
        String targetComponent = "bar";
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();

        // init mocks
        expect(identification.getName()).andReturn(suIdentifier).anyTimes();
        expect(su.getIdentification()).andReturn(identification).anyTimes();
        expect(su.getTargetArtifactsZip()).andReturn(artifactZip);
        expect(su.getTargetComponentName()).andReturn(targetComponent)
                .anyTimes();
        sus.add(su);
        try {
            deploymentManager.checkSUIdentifier(isA(List.class),
                    eq(suIdentifier));
            deploymentManager.checkTargetComponentAndZip(artifactZip,
                    suIdentifier, targetComponent);
            deploymentManager.checkSUZipInSAZip(artifactZip);
            expect(deploymentManager.isSUAlreadyDeployedOnTargetedComponent(su))
                    .andReturn(true);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        replay(su);
        replay(identification);
        replay(deploymentManager);

        // Run Test case
        try {
            deploymentManager.checkServiceUnits(sus);
            fail("Must throw an exception if su is already deployed to component");
        } catch (Exception e) {
            // Do nothing, it's ok !
        }
    }

    public void testCheckSAPackage() {
        // Create Mock Objects and init parameters
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        Method method4 = null;
        try {
            method1 = SADeploymentManager.class.getDeclaredMethod(
                    "isSANameAlreadyUsed", new Class[] {});
            method2 = SADeploymentManager.class.getDeclaredMethod(
                    "getServiceAssemblyName", new Class[] {});
            method3 = SADeploymentManager.class.getDeclaredMethod(
                    "checkConnections", new Class[] {List.class});
            method4 = SADeploymentManager.class.getDeclaredMethod(
                    "checkServiceUnits", new Class[] {List.class});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3,
            method4};

        SADeploymentManager deploymentManager = createMock(
                SADeploymentManager.class, mockedMethods);
        ServiceAssembly assembly = createMock(ServiceAssembly.class);
        URI saArchiveURI = null;
        try {
            saArchiveURI = new URI("file:/bar.zip");
        } catch (URISyntaxException e1) {
            e1.printStackTrace();
        }
        String saName = "foo";
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        List<Connection> connections = new ArrayList<Connection>();
        deploymentManager.assembly = assembly;
        deploymentManager.saArchiveURI = saArchiveURI;

        /*
         * Test case 1 : all is ok
         */
        // init mocks
        expect(assembly.getConnections()).andReturn(connections);
        expect(assembly.getServiceUnits()).andReturn(sus);
        expect(deploymentManager.getServiceAssemblyName()).andReturn(saName)
                .anyTimes();
        expect(deploymentManager.isSANameAlreadyUsed()).andReturn(false);
        try {
            deploymentManager.checkConnections(connections);
            deploymentManager.checkServiceUnits(sus);
        } catch (Exception e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        replay(assembly);
        replay(deploymentManager);

        // Run Test case 1
        try {
            deploymentManager.checkSAPackage();
        } catch (Exception e) {
            e.printStackTrace();
        }

        /*
         * Test case 2 : saName is null
         */
        reset(deploymentManager);
        // init mocks
        expect(deploymentManager.getServiceAssemblyName()).andReturn(null)
                .anyTimes();
        expect(deploymentManager.isSANameAlreadyUsed()).andReturn(false);
        replay(deploymentManager);

        // Run Test case 2
        try {
            deploymentManager.checkSAPackage();
            fail("Must throw an exception if sa name is null");
        } catch (Exception e) {
            // Do nothing it's ok !
        }

        /*
         * Test case 3 : saName is already used
         */
        reset(deploymentManager);
        // init mocks
        expect(deploymentManager.getServiceAssemblyName()).andReturn(saName)
                .anyTimes();
        expect(deploymentManager.isSANameAlreadyUsed()).andReturn(true);
        replay(deploymentManager);

        // Run Test case 3
        try {
            deploymentManager.checkSAPackage();
            fail("Must throw an exception if sa name is already used");
        } catch (Exception e) {
            // Do nothing it's ok !
        }
    }

    public void testCreateFrameworkTaskResult() {
        // Create Mock Objects and init parameters
        XMLResult result = createMock(XMLResult.class);
        String saName = "saName";
        SADeploymentManager deploymentManager = new SADeploymentManager();
        deploymentManager.xmlResult = result;

        /*
         * Test case 1 : deployment case
         */
        // init mocks
        result.addFrameworkTaskResult(eq(TaskResult.SUCCESS),
                eq(MessageType.INFO), contains("deployed"),
                aryEq(new String[] {saName}), eq(CauseFramework.YES));
        replay(result);

        // Run Test case 1
        try {
            deploymentManager.createFrameworkTaskResult(true, saName);
        } catch (Exception e) {
            e.printStackTrace();
        }

        /*
         * Test case 2 : undeployment case
         */
        reset(result);
        // init mocks
        result.addFrameworkTaskResult(eq(TaskResult.SUCCESS),
                eq(MessageType.INFO), contains("undeployed"),
                aryEq(new String[] {saName}), eq(CauseFramework.YES));
        replay(result);

        // Run Test case 2
        try {
            deploymentManager.createFrameworkTaskResult(false, saName);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testCreateSALifeCycle() {
        // Create Mock Objects and init parameters
        LifeCycleManagerService managerService = createMock(LifeCycleManagerService.class);
        URI installationRoot = null;
        try {
            installationRoot = new URI("file:/inst/");
        } catch (URISyntaxException e2) {
            e2.printStackTrace();
        }
        ObjectName depName = null;
        try {
            depName = new ObjectName("petals:int=deployment");
        } catch (MalformedObjectNameException e2) {
            e2.printStackTrace();
        } catch (NullPointerException e2) {
            e2.printStackTrace();
        }
        SADeploymentManager deploymentManager = new SADeploymentManager();
        LoggingUtil log = createMock(LoggingUtil.class);
        deploymentManager.log = log;
        deploymentManager.managerService = managerService;

        /*
         * Test case
         */
        // init mocks
        try {
            expect(managerService.getDeploymentServiceMBeanName()).andReturn(
                    depName);
        } catch (ManagementException e1) {
            e1.printStackTrace();
        }
        replay(managerService);

        // Run Test
        try {
            ServiceAssemblyLifeCycle lifeCycle = deploymentManager
                    .createSALifeCycle(installationRoot);
            assertNotNull(
                    "SADeploymentManager.createSALifeCycle must return a service assembly life cycle",
                    lifeCycle);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testExpendSUS() {
        // Create Mock Objects and init parameters
        Method method1 = null;
        Method method2 = null;
        Method method3 = null;
        try {
            method1 = SADeploymentManager.class.getDeclaredMethod(
                    "getSUArchiveURI", new Class[] {ServiceUnit.class,
                        URI.class});
            method2 = SADeploymentManager.class.getDeclaredMethod(
                    "expandSUIntoSA", new Class[] {String.class, URI.class,
                        String.class});
            method3 = SADeploymentManager.class.getDeclaredMethod(
                    "getServiceAssemblyName", new Class[] {});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1, method2, method3};
        SADeploymentManager deploymentManager = createMock(
                SADeploymentManager.class, mockedMethods);
        ServiceAssembly assembly = createMock(ServiceAssembly.class);
        ServiceUnit su = createMock(ServiceUnit.class);
        Identification identification = createMock(Identification.class);
        String suName = "foo";
        String saName = "bar";
        URI installationRoot = null;
        try {
            installationRoot = new URI("file:/inst/");
        } catch (URISyntaxException e2) {
            e2.printStackTrace();
        }
        URI suInstallationRoot = null;
        try {
            suInstallationRoot = new URI("file:/suinst/");
        } catch (URISyntaxException e2) {
            e2.printStackTrace();
        }
        URI suArchiveURI = null;
        try {
            suArchiveURI = new URI("file:/su.zip");
        } catch (URISyntaxException e2) {
            e2.printStackTrace();
        }
        LoggingUtil log = createMock(LoggingUtil.class);
        Map<String, URI> suInstallRoots = new HashMap<String, URI>();
        List<ServiceUnit> sus = new ArrayList<ServiceUnit>();
        sus.add(su);
        deploymentManager.log = log;
        deploymentManager.assembly = assembly;
        deploymentManager.suInstallRoots = suInstallRoots;

        /*
         * Test case 1 : all is ok
         */
        // init mocks
        expect(identification.getName()).andReturn(suName).anyTimes();
        expect(su.getIdentification()).andReturn(identification).anyTimes();
        expect(assembly.getServiceUnits()).andReturn(sus);
        expect(deploymentManager.getSUArchiveURI(su, installationRoot))
                .andReturn(suArchiveURI);
        expect(deploymentManager.getServiceAssemblyName()).andReturn(saName);
        try {
            expect(
                    deploymentManager.expandSUIntoSA(suName, suArchiveURI,
                            saName)).andReturn(suInstallationRoot);
        } catch (ManagementException e1) {
            e1.printStackTrace();
        }
        replay(deploymentManager);
        replay(identification);
        replay(su);
        replay(assembly);

        // Run Test case 1
        try {
            deploymentManager.expendSUS(installationRoot);
            Map<String, URI> suInstallRootsAfterExpend = deploymentManager.suInstallRoots;
            assertTrue("suInstallRoots map must contain the suName key",
                    suInstallRootsAfterExpend.containsKey(suName));
            assertTrue("suInstallRoots map must contain the su URI value",
                    suInstallRootsAfterExpend.containsValue(suInstallationRoot));
        } catch (Exception e) {
            e.printStackTrace();
        }

        /*
         * Test case 2 : SADeploymentManager.expandSUIntoSA throws an exception
         */
        reset(deploymentManager);
        reset(identification);
        reset(su);
        reset(assembly);
        // init mocks
        expect(identification.getName()).andReturn(suName).anyTimes();
        expect(su.getIdentification()).andReturn(identification).anyTimes();
        expect(assembly.getServiceUnits()).andReturn(sus);
        expect(deploymentManager.getSUArchiveURI(su, installationRoot))
                .andReturn(suArchiveURI);
        expect(deploymentManager.getServiceAssemblyName()).andReturn(saName);
        try {
            expect(
                    deploymentManager.expandSUIntoSA(suName, suArchiveURI,
                            saName)).andThrow(new ManagementException("excep"));
        } catch (ManagementException e1) {
            e1.printStackTrace();
        }
        replay(deploymentManager);
        replay(identification);
        replay(su);
        replay(assembly);

        // Run Test case 2
        try {
            deploymentManager.expendSUS(installationRoot);
            fail("Must throw an exception if SADeploymentManager.expandSUIntoSA throws an exception");
        } catch (Exception e) {
            // Do nothing, it's ok
        }
    }

    public void testExplodeSA() {
        // Create Mock Objects and init parameters
        Method method1 = null;
        try {
            method1 = SADeploymentManager.class.getDeclaredMethod(
                    "getServiceAssemblyName", new Class[] {});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};
        SADeploymentManager deploymentManager = createMock(
                SADeploymentManager.class, mockedMethods);
        PackageHandler packageHandler = createMock(PackageHandler.class);
        deploymentManager.packageHandler = packageHandler;
        String saName = "foo";
        URI saArcURI = null;
        URI saInstRoot = null;
        try {
            saArcURI = new URI("file:/sa.zip");
            saInstRoot = new URI("file:/inst");
        } catch (URISyntaxException e1) {
            e1.printStackTrace();
        }
        deploymentManager.saArchiveURI = saArcURI;

        /*
         * Test case
         */
        // init mocks
        expect(deploymentManager.getServiceAssemblyName()).andReturn(saName);
        try {
            expect(packageHandler.expandPackageIntoRepository(saName, saArcURI, EntityType.SA_TYPE))
                    .andReturn(saInstRoot);
        } catch (ManagementException e1) {
            e1.printStackTrace();
        }

        replay(deploymentManager);
        replay(packageHandler);

        // Run Test case 1
        try {
            URI result = deploymentManager.explodeSA();
            assertEquals("Wrong URI returned", saInstRoot, result);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testGetArchiveFile() {
        // Create init parameters
        SADeploymentManager deploymentManager = new SADeploymentManager();
        URI arcURI = null;
        try {
            arcURI = new URI("file:/foo");
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }
        File arcFile = new File(arcURI);

        // Launch test
        File result = deploymentManager.getArchiveFile(arcURI);
        assertEquals("Wrong file returned", arcFile, result);
    }

    public void testGetInstalledArchive() {
        // Create init parameters
        SADeploymentManager deploymentManager = new SADeploymentManager();
        File arcFile = new File("file:/foo/bar.zip");
        File installedFile = new File(AutoLoaderImpl.getInstalledDirectory(),
                arcFile.getName());

        // Launch test
        File result = deploymentManager.getInstalledArchive(arcFile);
        assertEquals("Wrong file returned", installedFile, result);
    }

    public void testGetSUArchiveURI() {
        // Create Mock Objects and init parameters
        SADeploymentManager deploymentManager = new SADeploymentManager();
        PackageHandler packageHandler = createMock(PackageHandler.class);
        ServiceUnit su = createMock(ServiceUnit.class);
        deploymentManager.packageHandler = packageHandler;
        String suZipNAme = "foo";
        URI suArcURI = null;
        URI saInstRoot = null;
        try {
            suArcURI = new URI("file:/su.zip");
            saInstRoot = new URI("file:/inst");
        } catch (URISyntaxException e1) {
            e1.printStackTrace();
        }

        /*
         * Test case
         */
        // init mocks
        expect(su.getTargetArtifactsZip()).andReturn(suZipNAme);
        expect(
                packageHandler.processAndGetPackageURI("file://"
                        + saInstRoot.getPath() + suZipNAme, true)).andReturn(
                suArcURI);

        replay(su);
        replay(packageHandler);

        // Run Test case 1
        try {
            URI result = deploymentManager.getSUArchiveURI(su, saInstRoot);
            assertEquals("Wrong URI returned", suArcURI, result);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testRegisterConnections() {
        // Create Mock Objects and init parameters
        Method method1 = null;
        try {
            method1 = SADeploymentManager.class.getDeclaredMethod(
                    "registerConnection", new Class[] {Connection.class});
        } catch (SecurityException e1) {
            e1.printStackTrace();
        } catch (NoSuchMethodException e1) {
            e1.printStackTrace();
        }
        Method[] mockedMethods = new Method[] {method1};
        SADeploymentManager deploymentManager = createMock(
                SADeploymentManager.class, mockedMethods);
        Connection connection = createMock(Connection.class);
        List<Connection> connections = new ArrayList<Connection>();
        connections.add(connection);
        ServiceAssembly assembly = createMock(ServiceAssembly.class);
        deploymentManager.assembly = assembly;

        /*
         * Test case
         */
        // init mocks
        expect(assembly.getConnections()).andReturn(connections);
        try {
            deploymentManager.registerConnection(connection);
        } catch (ManagementException e1) {
            e1.printStackTrace();
        }

        replay(assembly);
        replay(deploymentManager);

        // Run Test case 1
        try {
            deploymentManager.registerConnections();
            verify(deploymentManager);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void testRemoveConnections() {
        // Create Mock Objects and init parameters
        ServiceAssemblyLifeCycle assemblyLifeCycle = createMock(ServiceAssemblyLifeCycle.class);
        ServiceAssembly assembly = createMock(ServiceAssembly.class);
        EndpointService endpointService = createMock(EndpointService.class);
        Connection connection = createMock(Connection.class);
        List<Connection> connections = new ArrayList<Connection>();
        connections.add(connection);
        SADeploymentManager deploymentManager = new SADeploymentManager();
        deploymentManager.endpointService = endpointService;
        
        QName consInterface = new QName("foo");
        QName consService = new QName("foo2");
        String consEndpoint = "bar1";
        QName provService = new QName("foo3");
        String provEndpoint = "bar2";
        
        /*
         * Test case 1 : consInt is null
         */
        // init mocks
        expect(assemblyLifeCycle.getServiceAssembly()).andReturn(assembly);
        expect(assembly.getConnections()).andReturn(connections);
        expect(connection.getConsumerEndpointName()).andReturn(consEndpoint);
        expect(connection.getConsumerInterfaceName()).andReturn(null);
        expect(connection.getConsumerServiceName()).andReturn(consService);
        expect(connection.getProviderEndpointName()).andReturn(provEndpoint);
        expect(connection.getProviderServiceName()).andReturn(provService);
        try {
            endpointService.deleteConnection(consService, consEndpoint, provService, provEndpoint);
        } catch (JBIException e1) {
            e1.printStackTrace();
        }

        replay(assembly);
        replay(assemblyLifeCycle);
        replay(connection);
        replay(endpointService);

        // Run Test case 1
        try {
            deploymentManager.removeConnections(assemblyLifeCycle);
            verify(endpointService);
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        /*
         * Test case 2 : consInt not null
         */
        reset(assembly);
        reset(assemblyLifeCycle);
        reset(connection);
        reset(endpointService);
        // init mocks
        expect(assemblyLifeCycle.getServiceAssembly()).andReturn(assembly);
        expect(assembly.getConnections()).andReturn(connections);
        expect(connection.getConsumerEndpointName()).andReturn(consEndpoint);
        expect(connection.getConsumerInterfaceName()).andReturn(consInterface);
        expect(connection.getConsumerServiceName()).andReturn(consService);
        expect(connection.getProviderEndpointName()).andReturn(provEndpoint);
        expect(connection.getProviderServiceName()).andReturn(provService);
        try {
            endpointService.deleteConnection(consInterface, provService, provEndpoint);
        } catch (JBIException e1) {
            e1.printStackTrace();
        }

        replay(assembly);
        replay(assemblyLifeCycle);
        replay(connection);
        replay(endpointService);

        // Run Test case 2
        try {
            deploymentManager.removeConnections(assemblyLifeCycle);
            verify(endpointService);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
