/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: RouterInterceptorImpl.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.router;

import java.util.ArrayList;
import java.util.Collection;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.petals.jbi.component.ComponentContextImpl;
import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.jbi.routing.Router;
import org.objectweb.petals.jbi.routing.RoutingException;


/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass,<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 *
 * This class is used to intercept the message that we will send or receive 
 * and to start a monitoring-operation in a new Threads. 
 */
public class RouterInterceptorImpl implements Router,RouterInterceptor, BindingController{
    public static final int beforeSent = 0;
    public static final int afterSent = 1;
    public static final int beforereceived = 2;
    public static final int afterReceived = 3;
    
    private Router router;
    private Collection<RouterMonitor> monitors = new ArrayList<RouterMonitor>();
    
    private static final String[] CLIENT_INTERFACES = new String[] {
        "router"};
    
    
    /**
     * Send the message to the real router.
     * Monitors are called before and after this call (there are launched in new Threads).
     * @param : exchange
     * @param : source
     */
    public void send(ComponentContextImpl source, MessageExchangeImpl exchange) throws RoutingException {
      try{
          MessageExchangeImpl message = (MessageExchangeImpl) (exchange.clone());
          MessageExchangeImpl msg;
          for (RouterMonitor monitor : monitors) {
              msg = (MessageExchangeImpl) (message.clone());
              Thread tm= new ThreadMonitor(msg, monitor, RouterInterceptorImpl.beforeSent);
              tm.start();
          }
          router.send(source, exchange);
          
          for (RouterMonitor monitor : monitors) {
              msg = (MessageExchangeImpl) (exchange.clone());
              Thread tm= new ThreadMonitor(msg, monitor, RouterInterceptorImpl.afterSent);
              tm.start();
          }
      }catch(Exception e){
          e.printStackTrace();
          System.out.println("an error hapened, the message doesn't be sent");
      }
    }
    
    /**
     * Receive message from the Transporter
     * Monitors are called before and after this call (there are launched in new Threads).
     * @param : exchange
     * @param : source
     */
    public void receive(MessageExchangeImpl exchange) throws RoutingException {
        
        try{
            MessageExchangeImpl message = (MessageExchangeImpl) (exchange.clone());
            for (RouterMonitor monitor : monitors) {
                MessageExchangeImpl msg = (MessageExchangeImpl) (message.clone());
                Thread tm= new ThreadMonitor(msg, monitor, RouterInterceptorImpl.beforereceived);
                tm.start();
            }
            router.receive(exchange);
            for (RouterMonitor monitor : monitors) {
                MessageExchangeImpl msg = (MessageExchangeImpl) (message.clone());
                Thread tm= new ThreadMonitor(msg, monitor, RouterInterceptorImpl.afterReceived);
                tm.start();
            }
        }catch(Exception e){
            e.printStackTrace();
            System.out.println("an error hapened, the message doesn't be received");
        }
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#lookupFc(java.lang.String)
     */
    public Object lookupFc (String itfName) {
       if(itfName.equals("router"))
           return this.router;
       return null;
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String, java.lang.Object)
     */
    public void bindFc (String itfName, Object itfValue) {
        if(itfName.equals("router")){
            router = (Router)itfValue;
        }
    }
    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#unbindFc(java.lang.String)
     */
    public void unbindFc (String itfName) {
        if(itfName.equals("router")){
            router = null;
        }
    }

    /* (non-Javadoc)
     * @see org.objectweb.fractal.api.control.BindingController#listFc()
     */
    public String[] listFc() {
        return CLIENT_INTERFACES;
    }

    /**
     * Add new monitor to the monitor list for router 
     * @param : monitor to add
     * @return : result of add.
     */
    public boolean addMonitor(RouterMonitor monitor) {
        boolean add = this.monitors.add(monitor);
        return add;
    }

}
