/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: SupervisionMeter.java,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.router.meter;

import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.kernel.admin.ContainerInformation;
import org.objectweb.petals.kernel.admin.DistributedJMXServer;
import org.objectweb.petals.kernel.admin.DistributedJMXServerFactory;
import org.objectweb.petals.kernel.joram.JoramAgent;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass,<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 *
 *  Reports message meter 
 */
/**
 * @author oanass
 *
 */
public class MeterSupervisor implements MeterSupervisorMBean{
    
    private DistributedJMXServerFactory jmxFactory;
    private AdminAttributes adminAttributes;
    private JoramAgent joram;
    
    public MeterSupervisor(DistributedJMXServerFactory jmxFactory, JoramAgent joram, AdminAttributes adminAttributes) {
        this.jmxFactory = jmxFactory;
        this.adminAttributes = adminAttributes;
        this.joram = joram;
    }

  
    /**
     * @see org.objectweb.petals.monitoring.router.meter.MeterSupervisorMBean#getGlobalResult()
     */
    public String getGlobalResult(){
        int nbrMsg = 0;
        try{
//          Retrieve the ContainerInformation for all the launched containers. 
            List<ContainerInformation> listeContainerInformation = joram.retrieveAllContainersInformation();
            Iterator<ContainerInformation> containersIt = listeContainerInformation.iterator();
            while (containersIt.hasNext()) {
                try {
                    ContainerInformation information = containersIt.next();
                    DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(information.getName());
                    ObjectName result = null;
                    String pattern = "FC/Petals@*:itf=monitor-meter";
                    Set objNames;
                    ObjectName objName = new ObjectName(pattern);
                    objNames = server.queryNames(objName, null);
                    if (objNames != null && objNames.size() == 1) {
                        result = (ObjectName) objNames.iterator().next();
                    }
                    if (result != null)
                        nbrMsg += (Integer) server.invoke(result, "count", new Object[0], new String[0]);
                }catch (Exception e) {
                    e.printStackTrace();
                }
            }
            
        } catch (Exception e) {
            e.printStackTrace();
        }
        return("number of messages sent is : " + nbrMsg);
    }
    
    /**
     * @see org.objectweb.petals.monitoring.router.meter.MeterSupervisorMBean#getLocalResult()
     */
    public String getLocalResult(){
        int nbrMsg = 0;
        try{
            MBeanServerConnection connection = adminAttributes.getRawMBeanServer();
            ObjectName result = null;
            String pattern = "FC/Petals@*:itf=monitor-meter";
            Set objNames;
            ObjectName objName = new ObjectName(pattern);
            objNames = connection.queryNames(objName, null);
            if (objNames != null && objNames.size() == 1) {
                result = (ObjectName) objNames.iterator().next();
            }
            if (result != null)
                nbrMsg += (Integer) connection.invoke(result, "count", new Object[0], new String[0]);
        }catch (Exception e) {
            e.printStackTrace();
        }
        return("number of messages sent is : " + nbrMsg);
    }
    
    /**
     * called to store the counter to zero.
     */
    public void clear(){
        try{
//          Retrieve the ContainerInformation for all the launched containers. 
            List<ContainerInformation> listeContainerInformation = joram.retrieveAllContainersInformation();
            Iterator<ContainerInformation> containersIt = listeContainerInformation.iterator();
            while (containersIt.hasNext()) {
                try {
                    ContainerInformation information = containersIt.next();
                    DistributedJMXServer server = jmxFactory
                    .createDistributedJMXServer(information.getName());
                    ObjectName result = null;
                    String pattern = "FC/Petals@*:itf=monitor-meter";
                    Set objNames;
                    ObjectName objName = new ObjectName(pattern);
                    objNames = server.queryNames(objName, null);
                    if (objNames != null && objNames.size() == 1) {
                        result = (ObjectName) objNames.iterator().next();
                    }
                    if (result != null){
                        server.invoke(result, "clear", new Object[0], new String[0]);
                    }
                }catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
    }
}
