/*
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): OUAZZANI CHAHDI Anass
 * --------------------------------------------------------------------------
 * $Id: ContentMsg,v 1.2 2006/09/05 10:24:27 oanass Exp $
 * --------------------------------------------------------------------------  
 */

package org.objectweb.petals.monitoring.router.volume;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.jbi.messaging.NormalizedMessage;

import org.objectweb.petals.jbi.messaging.MessageExchangeImpl;
import org.objectweb.petals.monitoring.router.RouterMonitor;
import org.objectweb.petals.util.SourceHelper;

/**
 * @version $Rev: 617 $ $Date: 2006-06-19 17:28:41 +0200 (lun, 19 jun 2006) $
 * @since Petals 1.0
 * @author oanass,<a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 * 
 * Calculate the messages size :
 *  Summ of all the normalizedMessages of the Exchange 
 */
public class VolumeProbe implements RouterMonitor,Serializable {
    
    
    private static final long serialVersionUID = -1466370911446522100L;
    private List<Marker> markers = new ArrayList<Marker>();
    
    public List<Marker> getMarkers() {
        return markers;
    }

    /**
     * Clear the probe
     *
     */
    public synchronized void clear(){
        this.markers.clear();
    }
    
    /**
     *  Calculate the size of message exchange and add the result
     * to the list of marker whose each element contains an id of a message
     * and the size corresponding to this last. 
     * @param exchange
     */
    public synchronized void beforeMessageSent(MessageExchangeImpl _exchange) {
        long date = System.currentTimeMillis();
        
        Map<String, NormalizedMessage> listMessages =null;
        int size = 0;
        listMessages = _exchange.getMessages();
        Collection<NormalizedMessage> listNM = (Collection<NormalizedMessage>) listMessages.values();
        for (NormalizedMessage message : listNM) {
            if (message != null){
                String str;
                try {
                    str = SourceHelper.createString(message.getContent());
                    size += str.length();
                } catch (Exception e) {
                    //Do nothing;
                }
                
            }
        }
        this.markers.add(new Marker(_exchange.getExchangeId(), size, date));
    }
    
    
    /**
     * Do nothing
     * @param exchange
     */
    public synchronized void afterMessageSent(MessageExchangeImpl exchange) {
        //Do nothing
    }

    /**
     * Do nothing
     */
    public synchronized void beforeMessageReceived(MessageExchangeImpl message) {
        //Do nothing
        
    }

    /**
     * Do nothing
     */
    public synchronized void afterMessageReceived(MessageExchangeImpl message) {
        //Do nothing
        
    }
    
    /**
     * This class contains information about a message
     * (message id and size) 
     * @author oanass
     *
     */
    public class Marker implements Serializable{
        /**
         * 
         */
        private static final long serialVersionUID = 3130161625771314655L;
        String id;
        int size;
        long date;
        
        public String getId() {
            return id;
        }
        public int getSize() {
            return size;
        }
        
        public void setSize(int size) {
            this.size = size;
        }
        /**
         * 
         * @param id : id of message.
         * @param size : size of message.
         * 
         * The object Marker Contain the id of message that was sent
         * or received and her size.
         */
        public Marker(String id, int size, long date) {
            this.id = id;
            this.size = size;
            this.date = date;
        }
    }
}
