/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ServiceAssembly.java 12:14:20 rmarins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.tools.jbicommon.descriptor;

import java.util.List;


/**
 * The Service Assembly production is used to describe a service assembly
 * deployment package. This deployment descriptor describes the contained
 * service units, and to which components they are to be deployed.
 * 
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class ServiceAssembly {

    /**
     * Shared Library identification data.
     */
    private Identification identification;

    /**
     * Individual deployments to be made to components.
     */
    private List<ServiceUnit> serviceUnits;

    /**
     * Contains declarations of service interconnection metadata, used to map
     * static service consumers to service providers.
     */
    private List<Connection> connections;

    /**
     * Default constructor.
     */
    public ServiceAssembly() {
        super();
    }

    //
    // Getters and Setters
    //

    /**
     * Returns optional elements containing declarations of service
     * interconnection metadata, used to map static service consumers to service
     * providers.
     * 
     * @return <code>List<Connection></code> the interconnections between
     *         service consumers and providers mapping.
     */
    public List<Connection> getConnections() {
        return connections;
    }

    /**
     * Returns service assembly's identification data.
     * 
     * @return <code>Identification</code> the SA's identification.
     */
    public Identification getIdentification() {
        return identification;
    }

    /**
     * Returns service units elements describing the individual deployments to
     * be made to components.
     * 
     * @return <code>List<ServiceUnit></code> the individual deployments.
     */
    public List<ServiceUnit> getServiceUnits() {
        return serviceUnits;
    }

    /**
     * Set interconnections between service consumers and providers mapping.
     * 
     * @param connections
     *            <code>List<Connection></code> the interconnection mapping
     *            list.
     */
    protected void setConnections(List<Connection> connections) {
        this.connections = connections;
    }

    /**
     * Set the service assembly identification data.
     * 
     * @param identification
     *            The identification to set.
     */
    protected void setIdentification(Identification identification) {
        this.identification = identification;
    }

    /**
     * Set Service Assembly individual deployments.
     * 
     * @param serviceUnits
     *            <code>List<ServiceUnit></code> the individual deployments.
     */
    protected void setServiceUnits(List<ServiceUnit> serviceUnits) {
        this.serviceUnits = serviceUnits;
    }

}