package org.objectweb.petals.tools.jbicommon.descriptor;

import java.util.List;

/**
 * The services element is used to declare the static services provided and
 * consumed by a service unit. The term "static" in this context means those
 * services that are provided or consumed due to the deployment of the service
 * unit, and that are known at design- and/or deployment-time.
 * 
 * @version $Rev: 438 $ $Date: 2006-05-19 11:12:26Z $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public class Services {

    /**
     * Used to distinguish between service engine and binding component.
     */
    boolean bindingComponent;

    /**
     * Services provided by a service unit.
     */
    private List<Provides> provides;

    /**
     * Services consumed by a service unit.
     */
    private List<Consumes> consumes;

    /**
     * Default constructor.
     */
    public Services() {
        super();
    }

    //
    // Getters and Setters
    //

    /**
     * Attribute used to distinguish if the services declared is related to a
     * service engine or a binding component.
     * 
     * @return <code>boolean</code> indicates if it's a binding component.
     */
    public boolean isBindingComponent() {
        return bindingComponent;
    }

    /**
     * Returns the listing of services consumed by a deployed service unit.
     * 
     * @return <code>List<Consumes></code> the consumed services.
     */
    public List<Consumes> getConsumes() {
        return consumes;
    }

    /**
     * Returns the listing of services provided with the deployment of a service
     * unit.
     * 
     * @return <code>List<Consumes></code> the provided services.
     */
    public List<Provides> getProvides() {
        return provides;
    }

    /**
     * Set if it's relative to a binding component.
     * 
     * @param bindingComponent
     *            <code>true</code> to indicate it's a binding component.
     */
    void setBindingComponent(boolean bindingComponent) {
        this.bindingComponent = bindingComponent;
    }

    /**
     * Set the list of services consumed by a service unit deployed.
     * 
     * @param consumes
     *            the list of consumed services.
     */
    void setConsumes(List<Consumes> consumes) {
        this.consumes = consumes;
    }

    /**
     * Set the list of services provided with the deployment of a service unit.
     * 
     * @param provides
     *            the list of provided services.
     */
    void setProvides(List<Provides> provides) {
        this.provides = provides;
    }

}